/* gnuplot_i 2.x reference manual
 * http://ndevilla.free.fr/gnuplot/gnuplot_i/index.html
 */

include "std/dll.e" as dll
include "std/convert.e"
include "std/machine.e"
without warning

override function open_dll( sequence name )
    atom lib = dll:open_dll( name )
    ifdef DEBUG then
        printf( 1, "%s = #%08x\n", {name, lib} )
    end ifdef
    return lib
end function

override function define_c_func( object lib, object name, sequence params, integer return_type )
    integer id = dll:define_c_func( lib, name, params, return_type )
    ifdef DEBUG then
        printf( 1, "%s = %d\n", {name, id} )
    end ifdef
    return id
end function

override function define_c_proc( object lib, object name, sequence params )
    integer id = dll:define_c_proc( lib, name, params )
    ifdef DEBUG then
        printf( 1, "%s = %d\n", {name, id} )
    end ifdef
    return id
end function

atom libgnuplot = open_dll( "gnuplot_i.so" )

public constant
    xgnuplot_get_program_path	= define_c_func( libgnuplot, "gnuplot_get_program_path", {C_POINTER}, C_POINTER ),
    xgnuplot_init		= define_c_func( libgnuplot, "gnuplot_init", {}, C_POINTER ),
    xgnuplot_close		= define_c_proc( libgnuplot, "gnuplot_close", {C_POINTER} ),
    xgnuplot_cmd		= define_c_proc( libgnuplot, "gnuplot_cmd", {C_POINTER,C_POINTER} ),
    xgnuplot_setstyle		= define_c_proc( libgnuplot, "gnuplot_setstyle", {C_POINTER,C_POINTER} ),
    xgnuplot_set_xlabel		= define_c_proc( libgnuplot, "gnuplot_set_xlabel", {C_POINTER,C_POINTER} ),
    xgnuplot_set_ylabel		= define_c_proc( libgnuplot, "gnuplot_set_ylabel", {C_POINTER,C_POINTER} ),
    xgnuplot_resetplot		= define_c_proc( libgnuplot, "gnuplot_resetplot", {C_POINTER} ),
    xgnuplot_plot_x		= define_c_proc( libgnuplot, "gnuplot_plot_x", {C_POINTER,C_POINTER,C_INT,C_POINTER} ),
    xgnuplot_plot_xy		= define_c_proc( libgnuplot, "gnuplot_plot_xy", {C_POINTER,C_POINTER,C_POINTER,C_INT,C_POINTER} ),
    xgnuplot_plot_once		= define_c_proc( libgnuplot, "gnuplot_plot_once", {C_POINTER,C_POINTER,C_POINTER,C_POINTER,C_POINTER,C_POINTER,C_INT} ),
    xgnuplot_plot_slope		= define_c_proc( libgnuplot, "gnuplot_plot_slope", {C_POINTER,C_DOUBLE,C_DOUBLE,C_POINTER} ),
    xgnuplot_plot_equation	= define_c_proc( libgnuplot, "gnuplot_plot_equation", {C_POINTER,C_POINTER,C_POINTER} ),
    $

function allocate_float64_array( object data, integer count=-1 )

    -- make sure we have an array of values
    if atom( data ) then
        data = {data}
    end if

    -- auto-detect the array length, if necessary
    if count = -1 then
        count = length( data )
    end if

    -- allocate the block of memory
    atom ptr = allocate( count * 8 )

    -- poke the values into memory
    for i = 1 to count do
        poke( ptr + (i-1) * 8, atom_to_float64( data[i] ) )
    end for

    -- return the data pointer
    return ptr
end function

public type gnuplot_ctrl(atom x)
    return atom(x)
end type

public function gnuplot_get_program_path( sequence pname )
    atom pstr = c_func( xgnuplot_get_program_path, {allocate_string(pname, 1)} )
    return peek_string( pstr )
end function

public function gnuplot_init()
    return c_func( xgnuplot_init, {} )
end function

public procedure gnuplot_close( gnuplot_ctrl handle )
    c_proc( xgnuplot_close, {handle} )
end procedure

public procedure gnuplot_cmd( gnuplot_ctrl handle, sequence cmd )
    c_proc( xgnuplot_cmd, {handle, allocate_string(cmd, 1)} )
end procedure

public procedure gnuplot_setstyle( gnuplot_ctrl handle, sequence plot_style )
    c_proc( xgnuplot_setstyle, {handle, allocate_string(plot_style, 1)} )
end procedure

public procedure gnuplot_set_xlabel( gnuplot_ctrl handle, sequence label_x )
    c_proc( xgnuplot_set_xlabel, {handle, allocate_string(label_x, 1)} )
end procedure

public procedure gnuplot_set_ylabel( gnuplot_ctrl handle, sequence label_y )
    c_proc( xgnuplot_set_ylabel, {handle, allocate_string(label_y, 1)} )
end procedure

public procedure gnuplot_resetplot( gnuplot_ctrl handle )
    c_proc( xgnuplot_resetplot, {handle} )
end procedure

public procedure gnuplot_plot_x( gnuplot_ctrl handle, object d, integer n, sequence title )
    atom dptr = allocate_float64_array( d, n )
    c_proc( xgnuplot_plot_x, {handle, dptr, n, allocate_string(title, 1)} )
    free( dptr )
end procedure

public procedure gnuplot_plot_xy( gnuplot_ctrl handle, object x, object y, integer n, sequence title )
    atom xptr = allocate_float64_array( x, n )
    atom yptr = allocate_float64_array( y, n )
    c_proc( xgnuplot_plot_xy, {handle, xptr, yptr, n, allocate_string(title, 1)} )
    free( yptr )
    free( xptr )
end procedure

public procedure gnuplot_plot_once( sequence title, sequence style, sequence label_x, sequence label_y, object x, object y, integer n )
    atom xptr = allocate_float64_array( x, n )
    atom yptr = allocate_float64_array( y, n )
    c_proc( xgnuplot_plot_once, {allocate_string(title, 1), allocate_string(style, 1), allocate_string(label_x, 1), allocate_string(label_y, 1), xptr, yptr, n} )
    free( yptr )
    free( xptr )
end procedure

public procedure gnuplot_plot_slope( gnuplot_ctrl handle, atom a, atom b, sequence title )
    c_proc( xgnuplot_plot_slope, {handle, a, b, allocate_string(title, 1)} )
end procedure

public procedure gnuplot_plot_equation( gnuplot_ctrl handle, sequence equation, sequence title )
    c_proc( xgnuplot_plot_equation, {handle, allocate_string(equation, 1), allocate_string(title, 1)} )
end procedure

ifdef DEBUG then
    sequence path = gnuplot_get_program_path( "gnuplot" )
    printf( 1, "gnuplot = %s\n", {path} )
end ifdef
